@@ -262,13 +262,45 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
 
     @mock.patch.object(block_base.NetAppBlockStorageLibrary,
                        '_find_mapped_lun_igroup')
-    def test_unmap_lun(self, mock_find_mapped_lun_igroup):
-        mock_find_mapped_lun_igroup.return_value = (fake.IGROUP1_NAME, 1)
+    def test_unmap_lun_empty(self, mock_find_mapped_lun_igroup):
+        self.zapi_client.get_lun_map.return_value = fake.ISCSI_ONE_MAP_LIST
 
-        self.library._unmap_lun(fake.LUN_PATH, fake.FC_FORMATTED_INITIATORS)
+        self.library._unmap_lun(fake.LUN_PATH, fake.ISCSI_EMPTY_MAP_LIST)
 
-        self.zapi_client.unmap_lun.assert_called_once_with(fake.LUN_PATH,
-                                                           fake.IGROUP1_NAME)
+        mock_find_mapped_lun_igroup.assert_not_called()
+        self.zapi_client.get_lun_map.assert_called_once_with(fake.LUN_PATH)
+        self.zapi_client.unmap_lun.assert_called_once_with(
+            fake.LUN_PATH, fake.ISCSI_ONE_MAP_LIST[0]['initiator-group'])
+
+    @mock.patch.object(block_base.NetAppBlockStorageLibrary,
+                       '_find_mapped_lun_igroup')
+    def test_unmap_lun_detach_one(self, mock_find_mapped_lun_igroup):
+        fake_ini_group = fake.ISCSI_ONE_MAP_LIST[0]['initiator-group']
+        mock_find_mapped_lun_igroup.return_value = (fake_ini_group, 1)
+        self.zapi_client.get_lun_map.return_value = fake.ISCSI_ONE_MAP_LIST
+
+        self.library._unmap_lun(fake.LUN_PATH, fake.ISCSI_ONE_MAP_LIST)
+
+        mock_find_mapped_lun_igroup.assert_called_once_with(
+            fake.LUN_PATH, fake.ISCSI_ONE_MAP_LIST)
+        self.zapi_client.get_lun_map.assert_not_called()
+        self.zapi_client.unmap_lun.assert_called_once_with(
+            fake.LUN_PATH, fake_ini_group)
+
+    @mock.patch.object(block_base.NetAppBlockStorageLibrary,
+                       '_find_mapped_lun_igroup')
+    def test_unmap_lun_empty_detach_all(self, mock_find_mapped_lun_igroup):
+        self.zapi_client.get_lun_map.return_value = fake.ISCSI_MULTI_MAP_LIST
+
+        self.library._unmap_lun(fake.LUN_PATH, fake.ISCSI_EMPTY_MAP_LIST)
+
+        mock_find_mapped_lun_igroup.assert_not_called()
+        self.zapi_client.get_lun_map.assert_called_once_with(fake.LUN_PATH)
+        calls = [mock.call(fake.LUN_PATH,
+                           fake.ISCSI_MULTI_MAP_LIST[0]['initiator-group']),
+                 mock.call(fake.LUN_PATH,
+                           fake.ISCSI_MULTI_MAP_LIST[1]['initiator-group'])]
+        self.zapi_client.unmap_lun.assert_has_calls(calls)
 
     def test_find_mapped_lun_igroup(self):
         self.assertRaises(NotImplementedError,
